﻿using Kivii.Finances.Icbc;
using Kivii.Finances.Icbc.Entities;
using Kivii.Icbc.Entities;
using Kivii.Linq;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using Timer = System.Windows.Forms.Timer;

namespace Kivii.Client.Finances.Icbc
{
    public partial class FrmMain : Form
    {
        private JsonServiceClient _client = null;

        private Timer _timer = null;

        public FrmMain()
        {
            InitializeComponent();
        }

        #region 界面控件默认值管理
        private void loadDefault(params Control[] ctls)
        {
            using (var conn = KiviiContext.GetOpenedDbConnection<UiSetting>())
            {
                foreach (var ctl in ctls)
                {
                    var setting = conn.Select<UiSetting>(o => o.FormName == this.Name && o.ControlName == ctl.Name).FirstOrDefault();
                    if (setting == null) continue;
                    ctl.Text = setting.ControlValue;
                }
            }
        }

        private void setDefault(params Control[] ctls)
        {
            using (var conn = KiviiContext.GetOpenedDbConnection<UiSetting>())
            {
                foreach (var ctl in ctls)
                {
                    var setting = conn.Select<UiSetting>(o => o.FormName == this.Name && o.ControlName == ctl.Name).FirstOrDefault();
                    if (setting == null)
                    {
                        setting = new UiSetting();
                        setting.FormName = this.Name;
                        setting.ControlName = ctl.Name;
                        setting.ControlValue = ctl.Text;
                        conn.Insert(setting);
                    }
                    else
                    {
                        setting.ControlValue = ctl.Text;
                        conn.Update(setting);
                    }
                }
            }
        }

        private void initK5FormCtl(bool notLogin = true)
        {
            btnLogin.Enabled = notLogin;
            tbxServiceUrl.Enabled = notLogin;
            tbxUserName.Enabled = notLogin;
            tbxPassword.Enabled = notLogin;
            tbxNcUrl.Enabled = notLogin;
            tbxNcPort.Enabled = notLogin;
            nudxNcInterval.Enabled = notLogin;
            tbxNcCertId.Enabled = notLogin;
            tbxNcGroupId.Enabled = notLogin;
            tbxNcVersion.Enabled = notLogin;
            tbxNcAcc.Enabled = notLogin;
            //dtpCurrentTime.Enabled = notLogin;

            btnLogout.Enabled = !notLogin;
            btnNcSync.Enabled = !notLogin;
            btnNcStart.Enabled = !notLogin;
            btnNcStop.Enabled = !notLogin;
        }

        /// <summary>
        /// 输出消息
        /// </summary>
        /// <param name="message">消息内容</param>
        /// <param name="clearMessage">是否清除消息</param>
        private void outputMessage(string message, bool clearMessage = false)
        {
            if (this.InvokeRequired)
            {
                this.Invoke(new Action(() => outputMessage(message, clearMessage)));
                return;
            }
            if (clearMessage) rtbMessage.Clear();
            if (rtbMessage.Lines.Count() > 500) rtbMessage.Clear();
            var time = DateTime.Now.ToString("HH:mm:ss");
            rtbMessage.AppendText($"[{time}]{message}{Environment.NewLine}");
            rtbMessage.ScrollToCaret();
        }

        #endregion

        private void FrmMain_Load(object sender, EventArgs e)
        {
            initK5FormCtl();
            loadDefault(tbxServiceUrl, tbxUserName, tbxPassword, tbxNcAcc, tbxNcCertId, tbxNcGroupId, nudxNcInterval, tbxNcPort, tbxNcUrl, tbxNcVersion, dtpCurrentTime);
        }

        private void btnLogin_Click(object sender, EventArgs e)
        {
            if (_client != null)
            {
                MessageBox.Show("登陆前先退出原连接");
                return;
            }
            btnLogin.Enabled = false;

            _client = new JsonServiceClient(tbxServiceUrl.Text);
            var auth = new Authenticate { UserName = tbxUserName.Text, Password = tbxPassword.Text, provider = "Kivii" };
            var task = _client.PostAsync(auth);
            task.Success(resp =>
            {
                initK5FormCtl(false);
                setDefault(tbxServiceUrl, tbxUserName, tbxPassword, tbxNcAcc, tbxNcCertId, tbxNcGroupId, nudxNcInterval, tbxNcPort, tbxNcUrl, tbxNcVersion, dtpCurrentTime);
                outputMessage($"登录成功：{resp.FullName}");
                Thread.Sleep(100);
            }, true);
            task.Error(err =>
            {
                initK5FormCtl();
                _client.Dispose();
                _client = null;
                MessageBox.Show(err.Message, "Login Error");
            }, true);

        }
        private void btnLogout_Click(object sender, EventArgs e)
        {
            if (_client == null)
            {
                MessageBox.Show("未连接");
                return;
            }
            btnLogout.Enabled = false;

            var auth = new Authenticate { provider = "Logout" };
            var task = _client.PostAsync(auth);
            task.Success(resp =>
            {
                outputMessage($"退出成功");
                _client.Dispose();
                _client = null;
                Thread.Sleep(100);
                timerStop();
                initK5FormCtl();
            });
            task.Error(err =>
            {
                btnLogout.Enabled = true;
                MessageBox.Show(err.Message, "Logout Error");
            });
        }

        private void timerStart()
        {
            if (_client == null)
            {
                MessageBox.Show("请先登录！");
                return;
            }
            if (_timer == null)
            {
                _timer = new Timer();
                _timer.Interval = (int)(nudxNcInterval.Value * 60 * 1000);
                //_timer.Interval = (int)(nudxNcInterval.Value * 1000);
                _timer.Tick += _timer_Tick;
            }
            _timer.Start();
            btnNcStart.Enabled = false;
            btnNcStop.Enabled = true;
            outputMessage("定时器已启动");
        }

        private void timerStop()
        {
            if (_timer != null) _timer.Stop();
            _timer = null;
            btnNcStart.Enabled = true;
            btnNcStop.Enabled = false;
            outputMessage("定时器已停止");
        }

        private void btnNcStart_Click(object sender, EventArgs e)
        {
            timerStart();
        }

        private void btnNcStop_Click(object sender, EventArgs e)
        {
            timerStop();
        }

        private void btnNcSync_Click(object sender, EventArgs e)
        {
            executeSync();
        }

        private void _timer_Tick(object sender, EventArgs e)
        {
            try
            {
                _timer.Stop();
                executeSync();
            }
            catch (Exception ex)
            {
                outputMessage($"定时器错误：{ex.Message}");
            }
            finally
            {
                if (_timer != null) _timer.Start();
            }
        }


        private void executeSync()
        {
            var th = KiviiContext.NewThread(() =>
            {
                syncICBC();
            });
            th.Start();
        }
        private void syncICBC()
        {
            //if (InvokeRequired)
            //{
            //    this.Invoke(new Action(SyncPayments));
            //    return;
            //}
            if (!int.TryParse(tbxNcPort.Text, out int port))
            {
                outputMessage("NC端口号设置错误");
                return;
            }
            var ncConfig = new NetSafeConfig();
            ncConfig.HttpsIp = tbxNcUrl.Text;
            ncConfig.HttpsPort = port;
            ncConfig.GroupId = tbxNcGroupId.Text;
            ncConfig.CertId = tbxNcCertId.Text;
            ncConfig.InterfaceVersion = tbxNcVersion.Text;

            var icbc = new ICBCConfig();
            icbc.AccNo = tbxNcAcc.Text;
            icbc.MinAmt = "0";
            icbc.MaxAmt = "9999999999";

            ICBC_NetSafeClient nc1 = new ICBC_NetSafeClient();
            nc1.SetHttpsIP(ncConfig.HttpsIp);
            nc1.SetHttpsPort(ncConfig.HttpsPort);
            nc1.SetGroupID(ncConfig.GroupId);
            nc1.SetCertID(ncConfig.CertId);
            nc1.SetInterfaceVersion(ncConfig.InterfaceVersion);

            DateTime maxTranslateTime = dtpCurrentTime.Value;
            if (maxTranslateTime == DateTime.MinValue) maxTranslateTime = DateTime.Now;
            var beginTime = DateTime.Parse(maxTranslateTime.ToString("yyyy-MM-dd 00:00:00"));
            var endTime = DateTime.Parse(DateTime.Now.ToString("yyyy-MM-dd 23:59:59"));

            if (beginTime > endTime)
            {
                outputMessage($"收款数据中最后一次交易时间{beginTime}大于当前时间{endTime},数据有误!");
                return;
            }
            var items = nc1.GetICBCDetail(icbc, string.Empty, beginTime, endTime);

            if (items.IsNullOrEmpty())
            {
                outputMessage("未找到收款数据");
                return;
            }
            var request = new RequestAccept();
            request.Items = items;
            request.Icbc = icbc;
            try
            {
                var rtns = _client.Post(request);
                outputMessage($"New:查询日期：{beginTime:yyyy-MM-dd}成功查询{rtns.Total}条收款,插入{rtns.Insert}条,跳过重复{rtns.Skip}条!");
                DateTime maxTime = DateTime.MinValue;

                foreach (var item in items)
                {
                    var time = DateTime.Parse(DateTime.ParseExact(item.Time, "yyyy-MM-dd-HH.mm.ss.ffffff", CultureInfo.InvariantCulture).ToString("yyyy-MM-dd HH:mm:ss"));
                    if (time >= maxTime)
                    {
                        maxTime = time;
                    }
                }
                if (maxTime != DateTime.MinValue)
                {
                    this.Invoke(new Action(() =>
                    {
                        dtpCurrentTime.Value = maxTime;
                        setDefault(dtpCurrentTime);
                    }));
                }
            }
            catch (Exception ex)
            {
                var resp = ex.GetResponseStatus();
                if (resp == null) outputMessage($"请求同步流水失败，原因：{ex.Message}");
                else
                {
                    var msg = resp.Message;
                    outputMessage($"请求同步流水失败，原因：{msg}");
                }
            }
        }
    }
}
